@@ -397,7 +397,7 @@ lws_socket_bind(struct lws_vhost *vhost, struct lws *wsi,
 			lws_sa46_write_numeric_address((lws_sockaddr46 *)psin,
 							buf, sizeof(buf));
 
-			lwsl_wsi_notice(wsi, "source ads %s", buf);
+			lwsl_vhost_notice(vhost, "source ads %s", buf);
 		}
 
 	return port;
@@ -417,11 +417,13 @@ lws_retry_get_delay_ms(struct lws_context *context,
 		*conceal = 0;
 
 	if (retry) {
-		if (*ctry < retry->retry_ms_table_count)
-			ms = retry->retry_ms_table[*ctry];
-		else
-			ms = retry->retry_ms_table[
-				retry->retry_ms_table_count - 1];
+		if (retry->retry_ms_table_count) {
+			if (*ctry < retry->retry_ms_table_count)
+				ms = retry->retry_ms_table[*ctry];
+			else
+				ms = retry->retry_ms_table[
+					retry->retry_ms_table_count - 1];
+		}
 
 		/* if no percent given, use the default 30% */
 		if (retry->jitter_percent)
