@@ -111,7 +111,7 @@ _realloc(void *ptr, size_t size, const char *reason)
 
 	if (size) {
 #if defined(LWS_PLAT_FREERTOS)
-		lwsl_notice("%s: size %lu: %s (free heap %d)\n", __func__,
+		lwsl_debug("%s: size %lu: %s (free heap %d)\n", __func__,
 #if defined(LWS_AMAZON_RTOS)
 			    (unsigned long)size, reason, (unsigned int)xPortGetFreeHeapSize() - (int)size);
 #else
