@@ -66,21 +66,39 @@ lws_plat_dlopen(struct lws_plugin **pplugin, const char *libpath,
 	if (uv_dlsym(&lib, sym, &v)) {
 		uv_dlerror(&lib);
 		lwsl_err("%s: Failed to get '%s' on %s: %s\n",
-			 __func__, path, dent.name, lib.errmsg);
+			 __func__, path, libpath, lib.errmsg);
 		goto bail;
 	}
 
 	hdr = (const lws_plugin_header_t *)v;
 	if (hdr->api_magic != LWS_PLUGIN_API_MAGIC) {
-		lwsl_err("%s: plugin %s has outdated api %d (vs %d)\n",
+		lwsl_info("%s: plugin %s has outdated api %d (vs %d)\n",
 			 __func__, libpath, hdr->api_magic,
 			 LWS_PLUGIN_API_MAGIC);
 		goto bail;
 	}
 
+	if (strcmp(hdr->lws_build_hash, LWS_BUILD_HASH))
+		goto bail;
+
 	if (strcmp(hdr->_class, _class))
 		goto bail;
 
+	/*
+	 * We don't already have one of these, right?
+	 */
+
+	pin = *pplugin;
+	while (pin) {
+		if (!strcmp(pin->hdr->name, hdr->name))
+			goto bail;
+		pin = pin->list;
+	}
+
+	/*
+	 * OK let's bring it in
+	 */
+
 	pin = lws_malloc(sizeof(*pin), __func__);
 	if (!pin)
 		goto bail;
@@ -105,7 +123,9 @@ bail:
 int
 lws_plat_destroy_dl(struct lws_plugin *p)
 {
-	return uv_dlclose(&p->u.lib);
+	uv_dlclose(&p->u.lib);
+
+	return 0;
 }
 
 #endif
