@@ -227,11 +227,13 @@ read:
 	       // lwsi_state(wsi) != LRS_H1C_ISSUE_HANDSHAKE2 &&
 	       lwsi_state(wsi) != LRS_H2_WAITING_TO_SEND_HEADERS))) {
 
+		int scr_ret;
+
 		ebuf.token = pt->serv_buf;
-		ebuf.len = lws_ssl_capable_read(wsi,
+		scr_ret = lws_ssl_capable_read(wsi,
 					ebuf.token,
 					wsi->a.context->pt_serv_buf_size);
-		switch (ebuf.len) {
+		switch (scr_ret) {
 		case 0:
 			lwsl_info("%s: zero length read\n", __func__);
 			return LWS_HPI_RET_PLEASE_CLOSE_ME;
@@ -243,6 +245,19 @@ read:
 			return LWS_HPI_RET_PLEASE_CLOSE_ME;
 		}
 
+		/*
+		 * coverity is confused: it knows lws_ssl_capable_read may
+		 * return < 0 and assigning that to ebuf.len is bad, but it
+		 * doesn't understand this check below on scr_ret < 0
+		 * removes that possibility
+		 */
+
+		ebuf.len = scr_ret;
+		if (ebuf.len < 0) /* ie, not usable data */ {
+			lwsl_info("%s: other error\n", __func__);
+			return LWS_HPI_RET_PLEASE_CLOSE_ME;
+		}
+
 		// lwsl_notice("%s: Actual RX %d\n", __func__, ebuf.len);
 		// if (ebuf.len > 0)
 		//	lwsl_hexdump_notice(ebuf.token, ebuf.len);
@@ -313,7 +328,8 @@ drain:
 				lws_dll2_remove(&wsi->dll_buflist);
 			}
 		} else
-			if (n && n < ebuf.len && ebuf.len > 0) {
+			/* cov: both n and ebuf.len are int */
+			if (n > 0 && n < ebuf.len && ebuf.len > 0) {
 				// lwsl_notice("%s: h2 append seg %d\n", __func__, ebuf.len - n);
 				m = lws_buflist_append_segment(&wsi->buflist,
 						ebuf.token + n,
@@ -797,9 +813,9 @@ static int
 lws_h2_bind_for_post_before_action(struct lws *wsi)
 {
 	const struct lws_http_mount *hit;
+	int uri_len = 0, methidx;
 	char *uri_ptr = NULL;
 	uint8_t *buffered;
-	int uri_len = 0;
 	const char *p;
 	size_t blen;
 
@@ -850,7 +866,10 @@ lws_h2_bind_for_post_before_action(struct lws *wsi)
 		if (lws_bind_protocol(wsi, pp, __func__))
 			return 1;
 	}
-	if (lws_http_get_uri_and_method(wsi, &uri_ptr, &uri_len) >= 0)
+
+	methidx = lws_http_get_uri_and_method(wsi, &uri_ptr, &uri_len);
+
+	if (methidx >= 0)
 		if (wsi->a.protocol->callback(wsi, LWS_CALLBACK_HTTP,
 					      wsi->user_space,
 					      hit ? uri_ptr +
@@ -860,6 +879,10 @@ lws_h2_bind_for_post_before_action(struct lws *wsi)
 							  uri_len)))
 			return 1;
 
+#if defined(LWS_WITH_ACCESS_LOG)
+	lws_prepare_access_log_info(wsi, uri_ptr, uri_len, methidx);
+#endif
+
 	lwsl_info("%s: setting LRS_BODY from 0x%x (%s)\n", __func__,
 		    (int)wsi->wsistate, wsi->a.protocol->name);
 
