@@ -71,7 +71,6 @@ enum lejp_global_paths {
 
 static const char * const paths_vhosts[] = {
 	"vhosts[]",
-	"vhosts[].mounts[]",
 	"vhosts[].name",
 	"vhosts[].port",
 	"vhosts[].interface",
@@ -89,13 +88,17 @@ static const char * const paths_vhosts[] = {
 	"vhosts[].mounts[].auth-mask",
 	"vhosts[].mounts[].cgi-timeout",
 	"vhosts[].mounts[].cgi-env[].*",
+	"vhosts[].mounts[].cgi-env[]",
 	"vhosts[].mounts[].cache-max-age",
 	"vhosts[].mounts[].cache-reuse",
 	"vhosts[].mounts[].cache-revalidate",
 	"vhosts[].mounts[].basic-auth",
 	"vhosts[].mounts[].cache-intermediaries",
 	"vhosts[].mounts[].extra-mimetypes.*",
+	"vhosts[].mounts[].extra-mimetypes",
 	"vhosts[].mounts[].interpret.*",
+	"vhosts[].mounts[].interpret",
+	"vhosts[].mounts[]",
 	"vhosts[].ws-protocols[].*.*",
 	"vhosts[].ws-protocols[].*",
 	"vhosts[].ws-protocols[]",
@@ -108,6 +111,7 @@ static const char * const paths_vhosts[] = {
 	"vhosts[].ssl-option-set",
 	"vhosts[].ssl-option-clear",
 	"vhosts[].mounts[].pmo[].*",
+	"vhosts[].mounts[].pmo[]",
 	"vhosts[].headers[].*",
 	"vhosts[].headers[]",
 	"vhosts[].client-ssl-key",
@@ -140,7 +144,6 @@ static const char * const paths_vhosts[] = {
 
 enum lejp_vhost_paths {
 	LEJPVP,
-	LEJPVP_MOUNTS,
 	LEJPVP_NAME,
 	LEJPVP_PORT,
 	LEJPVP_INTERFACE,
@@ -158,13 +161,19 @@ enum lejp_vhost_paths {
 	LEJPVP_DEFAULT_AUTH_MASK,
 	LEJPVP_CGI_TIMEOUT,
 	LEJPVP_CGI_ENV,
+	LEJPVP_CGI_ENV_base,
 	LEJPVP_MOUNT_CACHE_MAX_AGE,
 	LEJPVP_MOUNT_CACHE_REUSE,
 	LEJPVP_MOUNT_CACHE_REVALIDATE,
 	LEJPVP_MOUNT_BASIC_AUTH,
 	LEJPVP_MOUNT_CACHE_INTERMEDIARIES,
 	LEJPVP_MOUNT_EXTRA_MIMETYPES,
+	LEJPVP_MOUNT_EXTRA_MIMETYPES_base,
 	LEJPVP_MOUNT_INTERPRET,
+	LEJPVP_MOUNT_INTERPRET_base,
+
+	LEJPVP_MOUNTS,
+
 	LEJPVP_PROTOCOL_NAME_OPT,
 	LEJPVP_PROTOCOL_NAME,
 	LEJPVP_PROTOCOL,
@@ -177,6 +186,7 @@ enum lejp_vhost_paths {
 	LEJPVP_SSL_OPTION_SET,
 	LEJPVP_SSL_OPTION_CLEAR,
 	LEJPVP_PMO,
+	LEJPVP_PM_baseO,
 	LEJPVP_HEADERS_NAME,
 	LEJPVP_HEADERS,
 	LEJPVP_CLIENT_SSL_KEY,
@@ -937,6 +947,8 @@ lwsws_get_config(void *user, const char *f, const char * const *paths,
 	struct lejp_ctx ctx;
 	int n, m = 0, fd;
 
+	memset(&ctx, 0, sizeof(ctx));
+
 	fd = lws_open(f, O_RDONLY);
 	if (fd < 0) {
 		lwsl_err("Cannot open %s\n", f);
