@@ -51,6 +51,8 @@
 
  SSL *SSL_SSL_from_mbedtls_ssl_context(mbedtls_ssl_context *msc);
 
+ mbedtls_ssl_context *SSL_mbedtls_ssl_context_from_SSL(SSL *ssl);
+
 /**
  * @brief create a SSL context
  *
@@ -58,7 +60,7 @@
  *
  * @return the context point
  */
-SSL_CTX* SSL_CTX_new(const SSL_METHOD *method);
+SSL_CTX* SSL_CTX_new(const SSL_METHOD *method, void *rngctx);
 
 /**
  * @brief free a SSL context
@@ -706,7 +708,7 @@ int SSL_CTX_get_verify_depth(const SSL_CTX *ctx);
  *
  * @return none
  */
-void SSL_CTX_set_verify(SSL_CTX *ctx, int mode, int (*verify_callback)(int, X509_STORE_CTX *));
+void SSL_CTX_set_verify(SSL_CTX *ctx, int mode, int (*verify_callback)(SSL *, mbedtls_x509_crt *));
 
 /**
  * @brief set the SSL verifying of the SSL context
@@ -717,7 +719,7 @@ void SSL_CTX_set_verify(SSL_CTX *ctx, int mode, int (*verify_callback)(int, X509
  *
  * @return none
  */
-void SSL_set_verify(SSL *s, int mode, int (*verify_callback)(int, X509_STORE_CTX *));
+void SSL_set_verify(SSL *s, int mode, int (*verify_callback)(SSL *, mbedtls_x509_crt *));
 
 /**
  * @brief set the SSL verify depth of the SSL context
@@ -737,7 +739,7 @@ void SSL_CTX_set_verify_depth(SSL_CTX *ctx, int depth);
  *
  * @return verifying result
  */
-int verify_callback(int preverify_ok, X509_STORE_CTX *x509_ctx);
+int verify_callback(SSL *, mbedtls_x509_crt *);
 
 /**
  * @brief set the session timeout time
