@@ -12,6 +12,9 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+
+#include "private-lib-core.h"
+
 #include "ssl_lib.h"
 #include "ssl_pkey.h"
 #include "ssl_x509.h"
@@ -19,8 +22,6 @@
 #include "ssl_dbg.h"
 #include "ssl_port.h"
 
-#include "private-lib-core.h"
-
 char *
 lws_strncpy(char *dest, const char *src, size_t size);
 
@@ -179,14 +180,19 @@ OSSL_HANDSHAKE_STATE SSL_get_state(const SSL *ssl)
     return state;
 }
 
+const char *mbedtls_client_preload_filepath;
+
 /**
  * @brief create a SSL context
  */
-SSL_CTX* SSL_CTX_new(const SSL_METHOD *method)
+SSL_CTX* SSL_CTX_new(const SSL_METHOD *method, void *rngctx)
 {
     SSL_CTX *ctx;
     CERT *cert;
     X509 *client_ca;
+#if defined(LWS_HAVE_mbedtls_x509_crt_parse_file)
+    int n;
+#endif
 
     if (!method) {
         SSL_DEBUG(SSL_LIB_ERROR_LEVEL, "no no_method");
@@ -199,7 +205,7 @@ SSL_CTX* SSL_CTX_new(const SSL_METHOD *method)
         goto failed1;
     }
 
-    cert = ssl_cert_new();
+    cert = ssl_cert_new(rngctx);
     if (!cert) {
         SSL_DEBUG(SSL_LIB_ERROR_LEVEL, "ssl_cert_new() return NULL");
         goto failed2;
@@ -214,9 +220,24 @@ SSL_CTX* SSL_CTX_new(const SSL_METHOD *method)
     ctx->method = method;
     ctx->client_CA = client_ca;
     ctx->cert = cert;
+    ctx->rngctx = rngctx;
 
     ctx->version = method->version;
 
+#if defined(LWS_HAVE_mbedtls_x509_crt_parse_file)
+    if (mbedtls_client_preload_filepath) {
+	mbedtls_x509_crt **px = (mbedtls_x509_crt **)ctx->client_CA->x509_pm;
+
+	*px = malloc(sizeof(**px));
+	mbedtls_x509_crt_init(*px);
+	n = mbedtls_x509_crt_parse_file(*px, mbedtls_client_preload_filepath);
+	if (n < 0)
+		lwsl_err("%s: unable to load cert bundle 0x%x\n", __func__, -n);
+	else
+		lwsl_info("%s: loaded cert bundle %d\n", __func__, n);
+    }
+#endif
+
     return ctx;
 
 failed3:
@@ -238,8 +259,10 @@ void SSL_CTX_free(SSL_CTX* ctx)
 
     X509_free(ctx->client_CA);
 
-    if (ctx->alpn_protos)
-	    ssl_mem_free(ctx->alpn_protos);
+    if (ctx->alpn_protos) {
+	    ssl_mem_free((void *)ctx->alpn_protos);
+	    ctx->alpn_protos = NULL;
+    }
 
     ssl_mem_free(ctx);
 }
@@ -294,7 +317,7 @@ SSL *SSL_new(SSL_CTX *ctx)
         goto failed2;
     }
 
-    ssl->cert = __ssl_cert_new(ctx->cert);
+    ssl->cert = __ssl_cert_new(ctx->cert, ctx->rngctx);
     if (!ssl->cert) {
         SSL_DEBUG(SSL_LIB_ERROR_LEVEL, "__ssl_cert_new() return NULL");
         goto failed3;
@@ -353,8 +376,10 @@ void SSL_free(SSL *ssl)
 
     SSL_SESSION_free(ssl->session);
 
-    if (ssl->alpn_protos)
-	    ssl_mem_free(ssl->alpn_protos);
+    if (ssl->alpn_protos) {
+	    ssl_mem_free((void *)ssl->alpn_protos);
+	    ssl->alpn_protos = NULL;
+    }
 
     ssl_mem_free(ssl);
 }
@@ -834,7 +859,7 @@ void SSL_CTX_set_default_read_buffer_len(SSL_CTX *ctx, size_t len)
 {
     SSL_ASSERT3(ctx);
 
-    ctx->read_buffer_len = len;
+    ctx->read_buffer_len = (int)len;
 }
 
 /**
@@ -845,7 +870,7 @@ void SSL_set_default_read_buffer_len(SSL *ssl, size_t len)
     SSL_ASSERT3(ssl);
     SSL_ASSERT3(len);
 
-    SSL_METHOD_CALL(set_bufflen, ssl, len);
+    SSL_METHOD_CALL(set_bufflen, ssl, (int)len);
 }
 
 /**
@@ -1055,7 +1080,7 @@ void SSL_set_verify_depth(SSL *ssl, int depth)
 /**
  * @brief set the SSL context verifying of the SSL context
  */
-void SSL_CTX_set_verify(SSL_CTX *ctx, int mode, int (*verify_callback)(int, X509_STORE_CTX *))
+void SSL_CTX_set_verify(SSL_CTX *ctx, int mode, int (*verify_callback)(SSL *, mbedtls_x509_crt *))
 {
     SSL_ASSERT3(ctx);
 
@@ -1066,7 +1091,7 @@ void SSL_CTX_set_verify(SSL_CTX *ctx, int mode, int (*verify_callback)(int, X509
 /**
  * @brief set the SSL verifying of the SSL context
  */
-void SSL_set_verify(SSL *ssl, int mode, int (*verify_callback)(int, X509_STORE_CTX *))
+void SSL_set_verify(SSL *ssl, int mode, int (*verify_callback)(SSL *, mbedtls_x509_crt *))
 {
     SSL_ASSERT3(ssl);
 
@@ -1167,7 +1192,7 @@ _openssl_alpn_to_mbedtls(struct alpn_ctx *ac, char ***palpn_protos)
 
 	/* allocate space for count + 1 pointers and the data afterwards */
 
-	alpn_protos = ssl_mem_zalloc((count + 1) * sizeof(char *) + ac->len + 1);
+	alpn_protos = ssl_mem_zalloc((unsigned int)(count + 1) * sizeof(char *) + ac->len + 1);
 	if (!alpn_protos)
 		return;
 
@@ -1175,7 +1200,7 @@ _openssl_alpn_to_mbedtls(struct alpn_ctx *ac, char ***palpn_protos)
 
 	/* convert to mbedtls format */
 
-	q = (unsigned char *)alpn_protos + (count + 1) * sizeof(char *);
+	q = (unsigned char *)alpn_protos + (unsigned int)(count + 1) * sizeof(char *);
 	p = ac->data;
 	count = 0;
 
