@@ -113,12 +113,12 @@ lws_context_init_server_ssl(const struct lws_context_creation_info *info,
 	if (vhost->tls.use_ssl)
 		lws_context_init_alpn(vhost);
 
-	/* check certs once a day */
+	/* check certs in a few seconds (after protocol init) and then once a day */
 
 	context->pt[0].sul_tls.cb = lws_sul_tls_cb;
 	__lws_sul_insert_us(&context->pt[0].pt_sul_owner[LWSSULLI_MISS_IF_SUSPENDED],
 			    &context->pt[0].sul_tls,
-			    (lws_usec_t)24 * 3600 * LWS_US_PER_SEC);
+			    (lws_usec_t)5 * LWS_US_PER_SEC);
 
 	return 0;
 }
