@@ -29,12 +29,12 @@ static struct lws_context *context;
 static struct lws_protocols protocols[] = {
 	/* first protocol must always be HTTP handler */
 
-	{ "http-only", lws_callback_http_dummy, 0, 0, },
+	{ "http-only", lws_callback_http_dummy, 0, 0, 0, NULL, 0 },
 	LWS_PLUGIN_PROTOCOL_DUMB_INCREMENT,
 	LWS_PLUGIN_PROTOCOL_MIRROR,
 	LWS_PLUGIN_PROTOCOL_LWS_STATUS,
 	LWS_PLUGIN_PROTOCOL_POST_DEMO,
-	{ NULL, NULL, 0, 0 } /* terminator */
+	LWS_PROTOCOL_LIST_TERM
 };
 
 /*
@@ -59,8 +59,6 @@ static const struct lws_http_mount mount_ziptest_uncomm = {
 	LWSMPRO_FILE,	/* origin points to a callback */
 	14,			/* strlen("/ziptest"), ie length of the mountpoint */
 	NULL,
-
-	{ NULL, NULL } // sentinel
 }, mount_ziptest = {
 	(struct lws_http_mount *)&mount_ziptest_uncomm,			/* linked-list pointer to next*/
 	"/ziptest",		/* mountpoint in URL namespace on this vhost */
@@ -80,7 +78,6 @@ static const struct lws_http_mount mount_ziptest_uncomm = {
 	8,			/* strlen("/ziptest"), ie length of the mountpoint */
 	NULL,
 
-	{ NULL, NULL } // sentinel
 }, mount_post = {
 	(struct lws_http_mount *)&mount_ziptest, /* linked-list pointer to next*/
 	"/formtest",		/* mountpoint in URL namespace on this vhost */
@@ -100,7 +97,6 @@ static const struct lws_http_mount mount_ziptest_uncomm = {
 	9,			/* strlen("/formtest"), ie length of the mountpoint */
 	NULL,
 
-	{ NULL, NULL } // sentinel
 }, mount = {
 	/* .mount_next */		&mount_post,	/* linked-list "next" */
 	/* .mountpoint */		"/",		/* mountpoint URL */
