@@ -12,4 +12,4 @@ extern int lifetime, reported;
 void foreign_timer_service(void *foreign_loop);
 void signal_cb(int signum);
 
-extern const struct ops ops_libuv, ops_libevent, ops_glib, ops_libev;
+extern const struct ops ops_libuv, ops_libevent, ops_glib, ops_libev, ops_sdevent, ops_uloop;
