@@ -100,7 +100,7 @@ int main(int argc, const char **argv)
 	info.options =
 		LWS_SERVER_OPTION_HTTP_HEADERS_SECURITY_BEST_PRACTICES_ENFORCE;
 	if ((p = lws_cmdline_option(argc, argv, "-t"))) {
-		info.count_threads = atoi(p);
+		info.count_threads = (unsigned int)atoi(p);
 		if (info.count_threads < 1 || info.count_threads > LWS_MAX_SMP)
 			return 1;
 	} else
@@ -108,7 +108,7 @@ int main(int argc, const char **argv)
 
 #if defined(LWS_WITH_TLS)
 	if (lws_cmdline_option(argc, argv, "-s")) {
-		info.options = LWS_SERVER_OPTION_DO_SSL_GLOBAL_INIT | LWS_SERVER_OPTION_REQUIRE_VALID_OPENSSL_CLIENT_CERT;
+		info.options = LWS_SERVER_OPTION_DO_SSL_GLOBAL_INIT;
 		info.ssl_cert_filepath = "localhost-100y.cert";
 		info.ssl_private_key_filepath = "localhost-100y.key";
 	}
