@@ -73,7 +73,7 @@ callback_raw_echo(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 
 	case LWS_CALLBACK_RAW_WRITEABLE:
 		lwsl_notice("LWS_CALLBACK_RAW_WRITEABLE\n");
-		lws_write(wsi, pss->buf, pss->len, LWS_WRITE_HTTP);
+		lws_write(wsi, pss->buf, (unsigned int)pss->len, LWS_WRITE_HTTP);
 		break;
 	default:
 		break;
@@ -83,8 +83,8 @@ callback_raw_echo(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 }
 
 static const struct lws_protocols protocols[] = {
-	{ "raw-echo", callback_raw_echo, sizeof(struct pss__raw_echo), 2048 },
-	{ NULL, NULL, 0, 0 }
+	{ "raw-echo", callback_raw_echo, sizeof(struct pss__raw_echo), 2048, 0, NULL, 0 },
+	LWS_PROTOCOL_LIST_TERM
 };
 
 void sigint_handler(int sig)
