@@ -52,14 +52,14 @@ cb(struct lejp_ctx *ctx, char reason)
 	*p = '\0';
 
 	if (reason & LEJP_FLAG_CB_IS_VALUE) {
-		p += lws_snprintf(p, p - end, "   value '%s' ", ctx->buf);
+		p += lws_snprintf(p, lws_ptr_diff_size_t(end, p), "   value '%s' ", ctx->buf);
 		if (ctx->ipos) {
 			int n;
 
-			p += lws_snprintf(p, p - end, "(array indexes: ");
+			p += lws_snprintf(p, lws_ptr_diff_size_t(end, p), "(array indexes: ");
 			for (n = 0; n < ctx->ipos; n++)
-				p += lws_snprintf(p, p - end, "%d ", ctx->i[n]);
-			p += lws_snprintf(p, p - end, ") ");
+				p += lws_snprintf(p, lws_ptr_diff_size_t(end, p), "%d ", ctx->i[n]);
+			p += lws_snprintf(p, lws_ptr_diff_size_t(end, p), ") ");
 		}
 		lwsl_notice("%s (%s)\r\n", buf,
 		       reason_names[(unsigned int)
@@ -102,7 +102,7 @@ main(int argc, char *argv[])
 	fd = 0;
 
 	while (n > 0) {
-		n = read(fd, buf, sizeof(buf));
+		n = (int)read(fd, buf, sizeof(buf));
 		if (n <= 0)
 			continue;
 
